/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.rave.portal.service;

import org.apache.rave.portal.model.PortalPreference;

import java.util.List;
import java.util.Map;

/**
 * Service for portal preferences
 */
public interface PortalPreferenceService {

    /**
     * Creates a Map of all {@link org.apache.rave.portal.model.PortalPreference}'s using the preference key as key for the Map.Entry
     *
     * @return Map of PortalPreference's
     */
    Map<String, PortalPreference> getPreferencesAsMap();

    /**
     * Gets a {@link org.apache.rave.portal.model.PortalPreference} by its key
     *
     * @param key unique name of the preference
     * @return PortalPreference if it exists, otherwise {@literal null}
     */
    PortalPreference getPreference(String key);

    /**
     * Saves a {@link org.apache.rave.portal.model.PortalPreference} with a single value.
     * If a PortalPreference already exists with this key, its value(s) will be overwritten.
     *
     * @param key   of the preference, e.g. {@literal title}
     * @param value of the preference, e.g. {@literal "Rave"}
     */
    void savePreference(String key, String value);

    /**
     * Saves a {@link org.apache.rave.portal.model.PortalPreference} with a List of values.
     * If a PortalPreference already exists with this key, its value(s) will be overwritten.
     *
     * @param key    of the preference, e.g. {@literal colors}
     * @param values List of values of the preference, e.g. {@literal red}, {@literal yellow}, {@literal blue}
     */
    void savePreference(String key, List<String> values);

    /**
     * Saves a {@link org.apache.rave.portal.model.PortalPreference}
     *
     * @param preference PortalPreference to save
     */
    void savePreference(PortalPreference preference);
}
